@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.003';
+our $VERSION = '1.004';
 
 use base 'Mutex';
 use Mutex::Util ();
@@ -26,7 +26,7 @@ sub CLONE {
 sub DESTROY {
     my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
 
-    $obj->unlock() if $obj->{ $pid };
+    syswrite($obj->{_w_sock}, '0'), $obj->{ $pid } = 0 if $obj->{ $pid };
 
     if ($obj->{'_init_pid'} eq $pid) {
         ($^O eq 'MSWin32')
@@ -58,11 +58,9 @@ sub new {
 
 sub lock {
     my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
-    return if $obj->{ $pid };
 
-    1 until sysread($obj->{_r_sock}, my($b), 1) || ($! && !$!{'EINTR'});
-
-    $obj->{ $pid } = 1;
+    sysread($obj->{_r_sock}, my($b), 1), $obj->{ $pid } = 1
+        unless $obj->{ $pid };
 
     return;
 }
@@ -72,11 +70,9 @@ sub lock {
 
 sub unlock {
     my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
-    return unless $obj->{ $pid };
 
-    1 until syswrite($obj->{_w_sock}, '0') || ($! && !$!{'EINTR'});
-
-    $obj->{ $pid } = 0;
+    syswrite($obj->{_w_sock}, '0'), $obj->{ $pid } = 0
+        if $obj->{ $pid };
 
     return;
 }
@@ -87,17 +83,12 @@ sub synchronize {
     );
     return unless ref($code) eq 'CODE';
 
-    # lock mutex
-    unless ($obj->{ $pid }) {
-        1 until sysread($obj->{_r_sock}, my($b), 1) || ($! && !$!{'EINTR'});
-        $obj->{ $pid } = 1;
-    }
+    # lock, run, unlock - inlined for performance
+    sysread($obj->{_r_sock}, my($b), 1), $obj->{ $pid } = 1
+        unless $obj->{ $pid };
 
     (defined wantarray) ? @ret = $code->(@_) : $code->(@_);
-
-    # unlock mutex
-    1 until syswrite($obj->{_w_sock}, '0') || ($! && !$!{'EINTR'});
-    $obj->{ $pid } = 0;
+    syswrite($obj->{_w_sock}, '0'), $obj->{ $pid } = 0;
 
     return wantarray ? @ret : $ret[-1];
 }
@@ -120,7 +111,7 @@ Mutex::Channel - Mutex locking via a pipe or socket
 
 =head1 VERSION
 
-This document describes Mutex::Channel version 1.003
+This document describes Mutex::Channel version 1.004
 
 =head1 DESCRIPTION
 
